import {async, ComponentFixture, TestBed} from '@angular/core/testing';

import {TileComponent} from './tile.component';
import {TilesService} from "../../services/tiles.service";
import {Component, DebugElement, ViewChild} from "@angular/core";
import {Tile, TileDefinition} from "../../model/tile";
import {By} from "@angular/platform-browser";
import {RouterTestingModule} from "@angular/router/testing";
import {of} from "rxjs";
import {BrowserAnimationsModule} from "@angular/platform-browser/animations";

// mock a host container so we can input bind to the unit under test
@Component({
  template: '<app-tile [tile]="tile"></app-tile>'
})
class HostComponent {
  tile: Tile = {
    area: 23.64,
    custom1: "VLASS1.2",
    custom2: "2019-07-12",
    custom3: "60% imaged",
    custom4: null,
    custom5: null,
    decMax: -36,
    decMin: -40,
    epoch: 1,
    factor: 1.73,
    firstEpochHalf: false,
    id: 1420,
    name: "T01t05",
    raMax: 37.5,
    raMin: 30,
    tier: 1,
    scans: []
  };
  @ViewChild(TileComponent, /* TODO: add static flag */ {}) tileComponent: TileComponent;
}

describe('TileComponent', () => {
  let component: HostComponent;
  let fixture: ComponentFixture<HostComponent>;
  let tilesServiceSpy: { getTileDefinition: jasmine.Spy }; // we'll use a spy in place of the real service

  beforeEach(async(() => {
    // create the spy to inject as a substitute for the real TileService
    const tilesServiceSpy = jasmine.createSpyObj('TilesService', ['getTileDefinition']);

    TestBed.configureTestingModule({
      declarations: [HostComponent, TileComponent],
      providers: [{provide: TilesService, useValue: tilesServiceSpy}],
      imports: [RouterTestingModule.withRoutes([]), BrowserAnimationsModule]
    })
      .compileComponents();
  }));

  beforeEach(() => {
    fixture = TestBed.createComponent(HostComponent);
    component = fixture.componentInstance;
    tilesServiceSpy = TestBed.get(TilesService); // we want to get the 'real' instance from our env
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('should set percentcomplete on init', () => {
    expect(component.tileComponent.percentComplete).toBeUndefined('starts undefined');
    fixture.detectChanges(); // ngOnInit
    expect(component.tileComponent.percentComplete).toEqual('60%', 'set percentCompete on init')
  });

  it('load tileDefinition on expanding the details (detailsExposed)', () => {
    let expectedReturn = of(new TileDefinition());
    tilesServiceSpy.getTileDefinition.and.returnValue(expectedReturn);

    expect(component.tileComponent.detailsExposed).toBeFalsy('detailsExposed starts false');

    fixture.detectChanges(); //onInit();

    let showMoreButtonDe: DebugElement = fixture.debugElement.query(By.css('.show_more_button'));
    let showMoreButtonEl: HTMLButtonElement = showMoreButtonDe.nativeElement;
    showMoreButtonEl.click();

    fixture.detectChanges();

    expect(component.tileComponent.detailsExposed).toBeTruthy('more button set detailsExposed to true');
    expect(tilesServiceSpy.getTileDefinition.calls.any()).toBeTruthy('called tile service');
    expect(component.tileComponent.tileDescription).toEqual(expectedReturn, 'fake tileDescription returned');
  });
});
